<?php

define('IN_AJAX', true);
$ajax =& new ajax_common();

require('./common.php');

$ajax->init();

// Handle "board disabled via ON/OFF trigger"
if (file_exists(BB_DISABLED))
{
	$ajax->ajax_die($bb_cfg['board_disabled_msg']);
}

// Load actions required modules
switch ($ajax->action)
{
	case 'view_post':
		require(INC_DIR .'bbcode.'. PHP_EXT);
	break;
}

// position in $ajax->valid_actions['xxx']
define('AJAX_AUTH', 0);  //  'guest', 'user', 'mod', 'admin'

$user->session_start();
$ajax->exec();

//
// Ajax
//
class ajax_common
{
	var $request  = array();
	var $response = array();

	var $valid_actions = array(
	//   ACTION NAME             AJAX_AUTH
		'edit_user_profile' => array('admin'),
		'view_post'         => array('guest'),
	// dj_maxx: add release order mod
		'order_vote'        => array('user'),
		'order_delete'      => array('admin'),
		'order_add'         => array('user'),
		'order_yes'         => array('user'),
		'order_abuse'       => array('user'),
		'view_comment'      => array('user'),
		'o_comment_add'     => array('user'),
	// end of

	// dj_maxx: add bonus points
	    	'create_bonus'        => array('admin'),
	    	'delete_bonus'        => array('admin'),
	   	'exchange_bonus'      => array('user'),
	// end of
		'check_name'        => array('guest'),
		'check_email'       => array('guest'),
		'check_pass'        => array('guest'),
	// dj_maxx: add news rotate
		'rotate_data'       => array('user'),
	// end of
	// dj_maxx: add slot machine
		'slot_machine'      => array('user'),
	// end of

	);

	var $action = null;

	/**
	*  Constructor
	*/
	function ajax_common ()
	{
		ob_start(array(&$this, 'ob_handler'));
		header('Content-Type: text/plain');
	}

	/**
	*  Perform action
	*/
	function exec ()
	{
		global $lang;

		// Exit if we already have errors
		if (!empty($this->response['error_code']))
		{
			$this->send();
		}

		// Check that requested action is valid
		$action = $this->action;

		if (!$action || !is_string($action))
		{
			$this->ajax_die('no action specified');
		}
		else if (!$action_params =& $this->valid_actions[$action])
		{
			$this->ajax_die('invalid action: '. $action);
		}

		// Auth check
		switch ($action_params[AJAX_AUTH])
		{
			// GUEST
			case 'guest':
				break;

			// USER
			case 'user':
				if (IS_GUEST)
				{
					$this->ajax_die($lang['Need_to_login_first']);
				}
				break;

			// MOD
			case 'mod':
				if (!(IS_MOD || IS_ADMIN))
				{
					$this->ajax_die($lang['Only_for_mod']);
				}
				$this->check_admin_session();
				break;

			// ADMIN
			case 'admin':
				if (!IS_ADMIN)
				{
					$this->ajax_die($lang['Only_for_admin']);
				}
				$this->check_admin_session();
				break;

			default:
				trigger_error("invalid auth type for $action", E_USER_ERROR);
		}

		// Run action
		$this->$action();

		// Send output
		$this->send();
	}

	/**
	*  Exit on error
	*/
	function ajax_die ($error_msg, $error_code = E_AJAX_GENERAL_ERROR)
	{
		$this->response['error_code'] = $error_code;
		$this->response['error_msg'] = $error_msg;

		$this->send();
	}

	/**
	*  Initialization
	*/
	function init ()
	{
		$this->request = $_POST;

		if (AJAX_CHARSET)
		{
			array_deep($this->request, 'ajax_decode');
		}

		$this->action  =& $this->request['action'];
	}

	/**
	*  Send data
	*/
	function send ()
	{
		$this->response['action'] = $this->action;

		if (DBG_USER && SQL_DEBUG && !empty($_COOKIE['sql_log']))
		{
			$this->response['sql_log'] = get_sql_log();
		}

		// sending output will be handled by $this->ob_handler()
		exit();
	}

	/**
	*  OB Handler
	*/
	function ob_handler ($contents)
	{
		if (DBG_USER)
		{
			if ($contents)
			{
				$this->response['raw_output'] = $contents;
			}
		}

		if (AJAX_CHARSET)
		{
			array_deep($this->response, 'ajax_encode');
		}

		$response_js = bb_json_encode($this->response);

		if (GZIP_OUTPUT_ALLOWED && !defined('NO_GZIP'))
		{
			if (UA_GZIP_SUPPORTED && strlen($response_js) > 2000)
			{
				header('Content-Encoding: gzip');
				$response_js = gzencode($response_js, 1);
			}
		}

		return $response_js;
	}

	/**
	*  Admin session
	*/
	function check_admin_session ()
	{
		global $user, $HTTP_POST_VARS;

		if (!$user->data['session_admin'])
		{
			if (empty($this->request['user_password']))
			{
				$this->prompt_for_password();
			}
			else
			{
				$login_args = array(
					'login_username' => $user->data['username'],
					'login_password' => $HTTP_POST_VARS['user_password'],  // $HTTP_POST_VARS - for compatibility with phpbb
				);
				if (!$user->login($login_args, true))
				{
					$this->ajax_die('Wrong password');
				}
			}
		}
	}

	/**
	*  Prompt for password
	*/
	function prompt_for_password ()
	{
		$this->response['prompt_password'] = 1;
		$this->send();
	}

	/**
	*  Edit user profile
	*/
	function edit_user_profile ()
	{
		global $db, $bb_cfg, $lang;

		if (!$user_id = intval($this->request['user_id']) OR !$profiledata = get_userdata($user_id))
		{
			$this->ajax_die('invalid user_id');
		}
		if (!$field = (string) $this->request['field'])
		{
			$this->ajax_die('invalid profile field');
		}

		$table = USERS_TABLE;
		$value = (string) $this->request['value'];

		switch ($field)
		{
			case 'user_regdate':
			case 'user_lastvisit':
				$tz = TIMENOW + (3600 * $bb_cfg['board_timezone']);
				if (($value = strtotime($value, $tz)) < $bb_cfg['board_startdate'] OR $value > TIMENOW)
				{
					$this->ajax_die('invalid date: '. $this->request['value']);
				}
				$this->response['new_value'] = bb_date($value);
				break;

			case 'ignore_srv_load':
				$value = ($this->request['value']) ? 0 : 1;
				$this->response['new_value'] = ($profiledata['user_level'] != USER || $value) ? $lang['NO'] : $lang['YES'];
				break;

			case 'u_up_total':
			case 'u_down_total':
			case 'u_up_release':
			case 'u_up_bonus':
				if (!IS_SUPER_ADMIN)
				{
					$this->ajax_die($lang['Only_for_super_admin']);
				}

				$table = BT_USERS_TABLE;
				$value = (float) $this->request['value'];

				foreach (array('KB'=>1,'MB'=>2,'GB'=>3,'TB'=>4) as $s => $m)
				{
					if (strpos($this->request['value'], $s) !== false)
					{
						$value *= pow(1024, $m);
						break;
					}
				}
				$value = sprintf('%.0f', $value);
				$this->response['new_value'] = humn_size($value, null, null, ' ');

				if (!$btu = get_bt_userdata($user_id))
				{
					require(INC_DIR .'functions_torrent.'. PHP_EXT);
					generate_passkey($user_id, true);
					$btu = get_bt_userdata($user_id);
				}
				$btu[$field] = $value;
				$this->response['update_ids']['u_ratio'] = (string) get_bt_ratio($btu);
				break;

			default:
				$this->ajax_die("invalid profile field: $field");
		}

		$value_sql = $db->escape($value, true);
		$db->query("UPDATE $table SET $field = $value_sql WHERE user_id = $user_id LIMIT 1");

		$this->response['edit_id'] = $this->request['edit_id'];
	}

	/**
	*  View post
	*/
	function view_post ()
	{
		global $user, $db, $lang;

		$post_id = (int) $this->request['post_id'];

		$sql = "
			SELECT
			  p.*,
			  h.post_html, IF(h.post_html IS NULL, pt.post_text, NULL) AS post_text,
			  pt.post_subject, pt.bbcode_uid,
			  f.auth_read
			FROM       ". POSTS_TABLE      ." p
			INNER JOIN ". POSTS_TEXT_TABLE ." pt ON(pt.post_id = p.post_id)
			 LEFT JOIN ". POSTS_HTML_TABLE ." h  ON(h.post_id = pt.post_id)
			INNER JOIN ". FORUMS_TABLE     ." f  ON(f.forum_id = p.forum_id)
			WHERE
			  p.post_id = $post_id
			LIMIT 1
		";

		if (!$post_data = $db->fetch_row($sql))
		{
			$this->ajax_die($lang['Topic_post_not_exist']);
		}

		// Auth check
		if ($post_data['auth_read'] == AUTH_REG)
		{
			if (IS_GUEST)
			{
				$this->ajax_die($lang['Need_to_login_first']);
			}
		}
		else if ($post_data['auth_read'] != AUTH_ALL)
		{
			$is_auth = auth(AUTH_READ, $post_data['forum_id'], $user->data, $post_data);
			if (!$is_auth['auth_read'])
			{
				$this->ajax_die($lang['Topic_post_not_exist']);
			}
		}

		$this->response['post_id']   = $post_id;
		$this->response['post_html'] = get_parsed_post($post_data);
	}



	function check_name()
	{
		global $db, $lang;
		$username = (string) $this->request['username'];
		if ($username == '') 
		{ 
			$this->response['html'] = '<img src="images/aff_cross.gif"><span class="leechmed bold">&nbsp;'.$lang['NO_INPUT_NAME'].'</span>'; 
		} 
		else 
		{
			$username = $db->escape($username);

			$query = "SELECT user_id
				FROM " . USERS_TABLE . " 
				WHERE username = '$username'" ;
			if ($row = $db->fetch_row($query))  
			{
				$this->response['html'] = '<img src="images/aff_cross.gif"><span class="leechmed bold">&nbsp;'.$lang['NAME_EXISTS'].'</span>';  
			}
			else
			{
				$this->response['html'] = '<img src="images/aff_tick.gif">';   
			}
		}
	}

	function check_email()
	{
		global $db, $lang;
		$email = (string) $this->request['email'];
		if ($email == '')
		{ 
			$this->response['html'] = '<img src="images/aff_cross.gif"><span class="leechmed bold">&nbsp;'.$lang['NO_INPUT_EMAIL'].'</span>';
		}
		else
		{
			$email_check = explode('.', $email);
			$email_len = strlen($email_check[1]);
			if ((!strpbrk($email, '@')) || (($email_len < 2) or ($email_len > 4))  )
			{ 
				$this->response['html'] = '<img src="images/aff_cross.gif"><span class="leechmed  bold">&nbsp;'.$lang['EMAIL_NOT_VALID'].'</span>';
			} 
			else
			{
				$email = $db->escape($email);
				$query = "SELECT user_id
			              FROM " . USERS_TABLE . " 
			              WHERE user_email = '$email'" ;
				if ($row = $db->fetch_row($query))
				{
				     $this->response['html'] = '<img src="images/aff_cross.gif"><span class="leechmed bold">&nbsp;'.$lang['EMAIL_EXISTS'].'</span>'; 
				}
				else
				{
				     $this->response['html'] = '<img src="images/aff_tick.gif">'; 
				}
			}
		}
	}

	function check_pass()
	{
		global $lang;
		$pass = (string) $this->request['pass'];
		$pass_confirm = (string) $this->request['pass_confirm'];
		if (($pass == '') || ($pass_confirm == ''))
		{ 
			$this->response['html'] = '<img src="images/aff_cross.gif"><span class="leechmed bold">&nbsp;'.$lang['PASS_INDEFINITE'].'</span>'; 
		}
		else
		{
			if ( $pass != $pass_confirm)
			{
				$this->response['html'] = '<img src="images/aff_cross.gif"><span class="leechmed bold">&nbsp;'.$lang['PASS_NOT_CONFIRM'].'</span>';  
			}
			else
			{
				$this->response['html'] = '<img src="images/aff_tick.gif"><span class="seedmed bold">&nbsp;'.$lang['PASS_CONFIRM'].'</span>';   
			}
		}
	}


// dj_maxx: add news rotate
	function rotate_data()
	{
		global $db, $lang, $bb_cfg;
		
		$mode = (string) $this->request['mode'];
		$page = (int) $this->request['page'];
		$page_prev = $page - 1;
		$page_next = $page + 1;

		
		if ($mode == 'latest_news')
		{		
			if ($bb_cfg['show_latest_news'] AND $news_forum_ids = $bb_cfg['latest_news_forum_id'])
			{
				$news_count = max($bb_cfg['latest_news_count'], 1);
				$start = abs(($page-1) * $news_count);			

				$data = $db->fetch_rowset("
					SELECT topic_id, topic_time, topic_title, topic_color
					FROM ". TOPICS_TABLE ."
					WHERE forum_id IN ($news_forum_ids)
					ORDER BY topic_time DESC
					LIMIT $start, $news_count
				");
				$news_got = $db->sql_numrows();

				$next_button = ($news_got == $news_count) ? "<a onclick=\"ajax.exec({action : 'rotate_data', mode: 'latest_news', page: $page_next}); return false;\" href=\"#\" title=\" 5 \" class=\"news_next\" id=\"rotate_data\"></a>" :
										 "<a href=\"#\" title=\" 5 \" class=\"news_next grayed\" id=\"rotate_data\"></a>";
				$back_button = ($page > 1) ? "<a onclick=\"ajax.exec({action : 'rotate_data', mode: 'latest_news', page: $page_prev}); return false;\" href=\"#\" title=\" 5 \" class=\"news_back\" id=\"rotate_data\"></a>" : 
								 "<a href=\"#\" title=\" 5 \" class=\"news_back grayed\" id=\"rotate_data\"></a>";
			

				$this->response['html'] = "
					<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">
					<tr>
					<td class=\"news_left\">&nbsp;</td>
					<td class=\"news_center vTop\">$back_button&nbsp;$next_button</td>
					<td class=\"news_right\">&nbsp;</td>
					</tr>
					</table>
					<div id=\"latest_news_fade\" style=\"position: absolute; top: 29px; left: 12px; z-index: 3; display: block; overflow: hidden; white-space: nowrap; text-overflow: ellipsis; -o-text-overflow: ellipsis; width: 96%;\">
					  <table cellspacing=\"0\" cellpadding=\"0\" width=\"100%\"> 
					  <tr> 
					  <td width=\"70%\"> 
					  <table cellpadding=\"0\" width=\"100%\">
				";

				foreach ($data as $row)
				{
					$news_time = create_date('d-M', $row['topic_time']);
					$news_topic_id = $row['topic_id'];
					$news_title = $row['topic_title'];
					$topic_url = TOPIC_URL;
	
					$this->response['html'] .= "		
	  					<tr> 
	  						<td class=\"line\"><span class=\"news_date\">$news_time</span><span class=\"news_title\"><a href=\"$topic_url$news_topic_id\">$news_title</a></span></td> 
	  					</tr> 
					";
	
				}

				$this->response['html'] .= "</table></table></div>";
				$this->response['mode'] = $mode;
			}
			else
			{
				ajax_die("   !");
			}

		}
		elseif ($mode == 'latest_releases')
		{
			if ($bb_cfg['show_latest_discussions'] AND $discussions_forum_ids = $bb_cfg['latest_discussions_forum_id'])
			{
				$discussions_count = max($bb_cfg['latest_discussions_count'], 1);
				$start = abs(($page-1) * $discussions_count);

				$data = $db->fetch_rowset("
					SELECT topic_id, topic_time, topic_title, topic_color
					FROM ". TOPICS_TABLE ."
					WHERE forum_id NOT IN ($discussions_forum_ids)
					AND   topic_dl_type = " . TOPIC_DL_TYPE_DL . "
					ORDER BY topic_time DESC
					LIMIT $start, $discussions_count
				");
				$discussions_got = $db->sql_numrows();

				$next_button = ($discussions_got == $discussions_count) ? "<a onclick=\"ajax.exec({action : 'rotate_data', mode: 'latest_releases', page: $page_next}); return false;\" href=\"#\" title=\" 5 \" class=\"news_next\" id=\"rotate_data\"></a>" :
										 "<a href=\"#\" title=\" 5 \" class=\"news_next grayed\" id=\"rotate_data\"></a>";
				$back_button = ($page > 1) ? "<a onclick=\"ajax.exec({action : 'rotate_data', mode: 'latest_releases', page: $page_prev}); return false;\" href=\"#\" title=\" 5 \" class=\"news_back\" id=\"rotate_data\"></a>" : 
								 "<a href=\"#\" title=\" 5 \" class=\"news_back grayed\" id=\"rotate_data\"></a>";

				$this->response['html'] = "
					<div style=\"float:right; padding-right:10px; margin-top: -34px;\">$back_button&nbsp;$next_button</div>
					<ul id=\"latest_releases_fade\">
				";

				
				foreach ($data as $row)
				{
					$discussion_time = create_date('d-M', $row['topic_time']);
					$discussion_topic_id = $row['topic_id'];
					$discussion_title = $row['topic_title'];
					$discussion_color = $row['topic_color'] ? "style=\"color:".$row['topic_color'].";\"" : "";
					$topic_url = TOPIC_URL;
	
					$this->response['html'] .= "		
						<li><a href=\"$topic_url$discussion_topic_id\"$discussion_color  class=\"med\">$discussion_title</a></li>
					";
	
				}


				$this->response['html'] .= "</ul>";
				$this->response['mode'] = $mode;


			}
			else
			{
				ajax_die("    !");
			}

		}
		elseif ($mode == 'latest_world_news')
		{

			if ($bb_cfg['show_latest_world_news'] AND $world_news_forum_ids = $bb_cfg['latest_world_news_forum_id'])
			{
				$world_news_count = max($bb_cfg['latest_world_news_count'], 1);
				$start = abs(($page-1) * $world_news_count);

				$data = $db->fetch_rowset("
					SELECT topic_id, topic_time, topic_title, topic_color
					FROM ". TOPICS_TABLE ."
					WHERE forum_id IN ($world_news_forum_ids)
					ORDER BY topic_time DESC
					LIMIT $start, $world_news_count
				");
				$world_news_got = $db->sql_numrows();

				$next_button = ($world_news_got == $world_news_count) ? "<a onclick=\"ajax.exec({action : 'rotate_data', mode: 'latest_world_news', page: $page_next}); return false;\" href=\"#\" title=\" 5 \" class=\"news_next\" id=\"rotate_data\"></a>" :
										 "<a href=\"#\" title=\" 5 \" class=\"news_next grayed\" id=\"rotate_data\"></a>";
				$back_button = ($page > 1) ? "<a onclick=\"ajax.exec({action : 'rotate_data', mode: 'latest_world_news', page: $page_prev}); return false;\" href=\"#\" title=\" 5 \" class=\"news_back\" id=\"rotate_data\"></a>" : 
								 "<a href=\"#\" title=\" 5 \" class=\"news_back grayed\" id=\"rotate_data\"></a>";


				$this->response['html'] = "
					<div style=\"float:right; padding-right:10px; margin-top: -34px;\">$back_button&nbsp;$next_button</div>
					<ul id=\"latest_world_news_fade\">
				";

				foreach ($data as $row)
				{
					$world_news_time = create_date('d-M', $row['topic_time']);
					$world_news_topic_id = $row['topic_id'];
					$world_news_title = $row['topic_title'];
					$world_news_color = $row['topic_color'] ? "style=\"color:".$row['topic_color'].";\"" : "";
					$topic_url = TOPIC_URL;
	
					$this->response['html'] .= "		
						<li><a href=\"$topic_url$world_news_topic_id\"$world_news_color  class=\"med\">$world_news_title</a></li>
					";
	
				}

				$this->response['html'] .= "</ul>";
				$this->response['mode'] = $mode;

			}
			else
			{
				ajax_die("     !");
			}

		}
		else
		{
			ajax_die(" !");
		}


		// ajax_die($this->response['html']);


	}
// end of

// dj_maxx: add slot machine
	function slot_machine() 
	{
		global $db, $lang, $bb_cfg, $userdata;

		$current_time = TIMENOW;		
		$user_id 	= $userdata['user_id'];
		$mode 	  	= (string) $this->request['booboo'];
		$game_type	= (int) $this->request['boobooboo'];
		
		$slotsTypes = array(
				'cherry' => array(2,5,10),
				'orange' => array(0,15,30),
				'prune' => array(0,40,50),
				'bell' => array(0,50,80),
				'bar1' => array(0,0,100),
				'bar2' => array(0,0,150),
				'bar3' => array(0,0,250),
				'seven' => array(0,0,500),
				'anybar' => array(0,0,8),
		);

		$slots = array(
				0 => array('orange','bell','orange','bar2','prune','orange',
				'bar3','prune','orange','bar1','bell','cherry','orange',
				'prune','bell','bar1','cherry','seven','orange','prune',
				'orange','bell','orange'),

				1 => array('cherry','prune','orange','bell','bar1','cherry','prune',
				'bar3','cherry','bell','orange','bar1','seven','cherry',
				'bar2','cherry','bell','prune','cherry','orange','cherry',
				'prune','orange'),

				2 => array('cherry','orange','bell','prune','bar2','cherry','prune',
				'orange','bar3','cherry','bell','orange','cherry','orange',
				'cherry','prune','bar1','seven','bell','cherry','cherry',
				'orange','bell'),
		);


		if ($mode == 'getspin')
		{

			$spin1 = mt_rand(0,22);
			$spin2 = mt_rand(0,22);
			$spin3 = mt_rand(0,22);

			//    (c           bb_bt_users)
			if ($game_type == 1)
			{
				$row = $db->fetch_row("SELECT u_up_total FROM ".BT_USERS_TABLE." WHERE user_id = $user_id");
				if ($db->sql_numrows() == 0)
				{
					ajax_die("      ,    .");
				}
				elseif ($row['u_up_total'] >= 4*1024*1024*1024)
				{ 
					$db->query("UPDATE ".BT_USERS_TABLE." SET u_up_total = u_up_total - (4*1024*1024*1024) WHERE user_id = $user_id");
				}
				else
				{
					ajax_die(" , ,  !  !");
				}
			}
			elseif ($game_type == 2)
			{
				$row = $db->fetch_row("SELECT user_bonus FROM ".BT_USERS_TABLE." WHERE user_id = $user_id");
				if ($db->sql_numrows() == 0)
				{
					ajax_die("      -,    .");
				}
				if ($row['user_bonus'] >= 4)
				{ 
					$db->query("UPDATE ".BT_USERS_TABLE." SET user_bonus = user_bonus - 4 WHERE user_id = $user_id");
				}
				else
				{
					ajax_die(" , ,  !  !");
				}
			}
			else
			{
				ajax_die(" , , !");
			}


			$db->query("INSERT INTO bb_slot_machine (user_id, spin1, spin2, spin3, spin_time) VALUES ($user_id, $spin1, $spin2, $spin3, $current_time)
  					ON DUPLICATE KEY UPDATE spin1 = $spin1, spin2 = $spin2, spin3 = $spin3, spin_time = $current_time");

			$this->response['spin'][0] = $spin1;
			$this->response['spin'][1] = $spin2;
			$this->response['spin'][2] = $spin3;

		}

		elseif ($mode == 'getwin')
		{

			if ($row = $db->fetch_row("SELECT spin1, spin2, spin3, spin_time FROM bb_slot_machine WHERE user_id = $user_id"))
			{
				//    -   !
				$db->query("DELETE FROM bb_slot_machine WHERE user_id = $user_id");

				$spin1 = $row['spin1'];
				$spin2 = $row['spin2'];
				$spin3 = $row['spin3'];

				$winnedCredits = 0;
				$slotType = $slots[0][$spin1];
				$matches = 1;
				$barMatch = preg_match("/bar/", $slotType) ? 1 : 0;
					
				if ( $slotType == $slots[1][$spin2]) 
				{
					$matches++;
					if ($slotType == $slots[2][$spin3]) 
					{
						$matches++;
					} 
					elseif ($barMatch != 0 && preg_match("/bar/", $slots[2][$spin3])) 
					{
						$barMatch++;
					}
				} 
				elseif ($barMatch != 0 && preg_match("/bar/", $slots[1][$spin2])) 
				{
					$barMatch++;
					if (preg_match("/bar/", $slots[2][$spin3])) 
					{
						$barMatch++;
					}
				}
				if ($matches != 3 && $barMatch == 3) 
				{
					$slotType = 'anybar';
					$matches = 3;
				}
				
				$winnedCredits = (int) $slotsTypes[$slotType][$matches-1];

				//  
				if ($winnedCredits > 0)
				{
					if ($game_type == 1)
					{
						$db->query("UPDATE ".BT_USERS_TABLE." SET u_up_total = u_up_total + ($winnedCredits*1024*1024*1024) WHERE user_id = $user_id");
						if ($db->sql_affectedrows() == 0)
						{
							ajax_die(" ,   ,   , !");
						}
					}
					elseif ($game_type == 2)
					{
						$db->query("UPDATE ".BT_USERS_TABLE." SET user_bonus = user_bonus + $winnedCredits WHERE user_id = $user_id");
						if ($db->sql_affectedrows() == 0)
						{
							ajax_die(" ,   ,   , !");
						}
					}
					else
					{
						ajax_die("  , !");
					}			
					
				}

				$this->response['winnedCredits'] = $winnedCredits;

				// ajax_die(": " . $winnedCredits);
			}
			else
			{
				ajax_die("!!!");
			}


		}
		else
		{

			ajax_die(" ,    !!!");
		}
		



	}
// end of


// dj_maxx: add bonus points
	function create_bonus() 
	{
		global $db, $lang, $bb_cfg;
		$bonus_points = (int) $this->request['bonus_points'];
		$bonus_add    = (int) $this->request['bonus_add'];
		$bonus_name   = $lang['bonus_GB'];
		// test values
		if ((!$bonus_points) || (!$bonus_add)) 
		{
			ajax_die($lang['bonus_fill_fields']);
		}

		$max_id = $db->sql_query("SELECT MAX(bonus_id) as id FROM " . BONUS_TABLE);
		while ($max = $db->sql_fetchrow($max_id))
		{
			$maxid = $max['id'];
		}
		$db->query("INSERT bb_bonus SET 
		            bonus_points = $bonus_points,
		            bonus_name   = '$bonus_name',
		            bonus_id     = $maxid + 1,
		            bonus_add    = $bonus_add
	        ");

		$this->response['bid']  = $maxid + 1;
		$this->response['html'] = '<td>'.$bonus_add.'&nbsp;'.$bonus_name.'</td><td class="row1">'.$bonus_points.'</td>';

	}


	function delete_bonus() 
	{
		global $db, $lang, $bb_cfg;
		$bonus_id = (int) $this->request['bonus_id'];
		$sql = "DELETE FROM " . BONUS_TABLE . " WHERE bonus_id = " . $bonus_id;
		if(!$result = $db->sql_query($sql))
		{
			ajax_die('Could not delete bonus information');
		}
		$this->response['bid'] = $bonus_id;
	}


	function exchange_bonus()
	{
		global $db, $lang, $bb_cfg, $userdata;
		$user_id      = $userdata['user_id'];
		$user_name    = $userdata['username'];
		$bonus_id     = (int) $this->request['bonus_id'];
		$my_bonus     = (int) $this->request['my_bonus'];
		$bonus_magic  = $this->request['bonus_magic'];

		if (!$bonus_id) ajax_die($lang['bonus_need_select']);


		if ($bonus_magic)
		{
			ajax_die("!");
		}



		$bonus = $db->sql_query("SELECT * FROM " . BONUS_TABLE . " WHERE bonus_id = $bonus_id");

		while ($b_data = $db->sql_fetchrow($bonus))
		{
			$bonus_points = (int) $b_data['bonus_points'];
			$bonus_add    = (int) $b_data['bonus_add'];
		}
		if ($bonus_points > $my_bonus)
		{
			$this->response['html'] = '
				   <table class="error">
				   <td class="msg">
				      '.$lang['bonus_respected'].' <b>'.$user_name.'</b> !!  '.$lang['bonus_no_cheat'].' !<br /><br/>
				   <a href="bonus.php">'.$lang['bonus_back'].'</a>
				   </td>
				   </table>
				   ';
		} 
		else if (($bonus_points < $my_bonus) || ($bonus_points == $my_bonus)) 
		{

			// dj_maxx: BUG fix -    
			$row = $db->fetch_row("SELECT user_bonus FROM " . BT_USERS_TABLE . " WHERE user_id = $user_id LIMIT 1");
			if (intval($row['user_bonus']) != $my_bonus)
			{
				ajax_die("      ,      !");
			}
			else
			{
			//
				$bonus_add_byte = $bonus_add * 1073741824;
				$db->query("UPDATE ".BT_USERS_TABLE." SET 
		                		user_bonus = user_bonus - $bonus_points,
		          	      		u_up_total = u_up_total + $bonus_add_byte
				    	WHERE user_id = $user_id 
			            	LIMIT 1
			           ");

				$this->response['html'] = '
					   <table class="info_msg">
					   <td class="msg">
					      '.$lang['bonus_respected'].' <b>'.$user_name.'</b>, '.$lang['bonus_upload_added'].' '.$bonus_add.'.<br /><br/>
					      <a href="bonus.php">'.$lang['bonus_back'].'</a><br/><br/><a href="index.php">'.$lang['bonus_to_main'].'</a>
					   </td>
					   </table>
				   ';
			// dj_maxx: BUG fix -    
			}
			// end of
		}


	}
// end of



// dj_maxx: add release order mod

function order_vote()
{
	global $db;
	$id = (int) $this->request['id'];
	$db->query("UPDATE bb_order SET order_vote = order_vote+1 WHERE  order_id = $id");
	$vote_summ = $db->sql_query("SELECT order_vote FROM bb_order WHERE order_id = $id LIMIT 1");
	while ($votes = $db->sql_fetchrow($vote_summ))
	{     
		$votes_summ = $votes['order_vote'];
	}
	$this->response['vote'] = $votes_summ;
	$this->response['id'] = $id;
}

function order_delete()
{
	global $db;
	$id = (int) $this->request['id'];
	$db->query("DELETE FROM bb_order WHERE order_id = $id");
	$this->response['id'] = $id;
}

function order_add()
{
	global $db, $userdata, $bb_cfg, $lang;
	$name = (string) $this->request['name'];
	$desc = (string) $this->request['desc'];
	$f_id = (int) $this->request['forum'];
	if (($name == '') || ($desc == '')) ajax_die('Error');
	$time = TIMENOW;
	$user_id = $userdata['user_id'];
	$name = $db->escape($name);
	$desc = $db->escape($desc);
	$vote = 0;
	$html = '';
	$row = $db->fetch_row("SELECT forum_name FROM ". FORUMS_TABLE ." WHERE forum_id = ".$f_id);

	$db->query("INSERT bb_order SET 
               order_forum_id = $f_id,
               order_name = '$name',
               order_desc = '$desc',
               order_time = $time,
               order_yes  = 0,
               order_user_id = $user_id,
               order_vote = $vote
        ");

	$html .= '<td class="small bold w10 tCenter"><a href="viewforum.php?f='.$f_id.'">'.$row['forum_name'].'</a></td>';
	$html .= '<td class="bold w30" name="hilite">'.$name.'</td>';
	$html .= '<td class="w10 tCenter">'.create_date($bb_cfg['default_dateformat'], $time, $bb_cfg['board_timezone']).'</td>';
	$html .= '<td class="bold w10 tCenter"><a href="profile.php?mode=viewprofile&u='.$user_id.'">'.$userdata['username'].'</a></td>';
	$html .= '<td class="w10 tCenter">'.$lang['NO'].'</td>';
	$html .= '<td class="bold w10 tCenter">----</td>';
	$html .= '<td class="w10 tCenter bold">0</td>';
	$html .= '<td class="w10 tCenter bold"><img src="'.BB_ROOT.'images/order_normal.png" /></td>';

	if ($userdata['user_level'] == ADMIN)
	{
		$html .= '<td class="w10 bold tCenter leech">X</td>';
	}

	$this->response['html'] = $html;
}


function order_yes()
{
	global $db, $userdata, $lang;
	$id = (int) $this->request['id'];
	$topic_id = (int) $this->request['topic_id'];
	$user_id = $userdata['user_id'];
	$db->query("UPDATE bb_order SET order_yes = 1, order_user_performed_id = $user_id, order_topic_id = $topic_id WHERE order_id = $id");
	$this->response['id'] = $id;
	$this->response['username'] = '<a href="profile.php?mode=viewprofile&u='.$user_id.'">'.$userdata['username'].'</a>';
	$this->response['html'] = '<a href="viewtopic.php?t='.$topic_id.'" class="seed bold" />'.$lang['YES'].'</b>';
}

function order_abuse()
{
	global $db;
	$id = (int) $this->request['id'];
	$db->query("UPDATE bb_order SET order_abuse = 1 WHERE order_id = $id");
	$this->response['id'] = $id;
}

function view_comment()
{
	global $db, $userdata, $lang, $bb_cfg;
	$order_id = (int) $this->request['or_id'];
	$user_id = $userdata['user_id'];
	$i = 0;
	$comments = $db->query("SELECT c.*, u.username 
              FROM bb_order_comment AS c
              LEFT JOIN bb_users AS u ON(u.user_id = c.comment_user_id)
              WHERE order_id = $order_id
              ORDER BY comment_time
        ");
	$html = '<table class="forumline w100">';
	while ($com = $db->sql_fetchrow($comments))
	{     
		$i++;
		$row_class = !($i % 2) ? 'row2' : 'row1';
		$html .= '<tr class="'.$row_class.'">';
		$html .= '<td width="10%" class="tCenter bold"><a href="profile.php?mode=viewprofile&u='.$user_id.'">'.$com['username'].'</a><br />'.create_date($bb_cfg['default_dateformat'], $com['comment_time'], $bb_cfg['board_timezone']).'</td>';
		$html .= '<td width="90%">'.htmlspecialchars($com['comment']).'</td>';
		$html .= '</tr>';
	}

	$html .= '<tr class="row3">';
	$html .= '<td class="w100 bold tCenter" colspan="2"> </td>';
	$html .= '</tr>';
	$html .= '<tr class="row1">';
	$html .= '<td width="20%" class="bold"><br /><span class="small"> <b>BBCode</b>  <b>HTML</b> </span></td>';
	$html .= '<td width="80%" class="bold tCenter"><textarea id="ctext_'.$order_id.'" style="width: 90%; height: 100px;"></textarea></td>';
	$html .= '</tr>';
	$html .= '<tr>';
	$html .= '<td colspan="2" class="tCenter"><input type="button" onclick="ajax.o_comment_add('.$order_id.');" value="" /></td>';
	$html .= '</tr>';
	$html .= '</table>';

	$this->response['html']  = $html;
	$this->response['or_id'] = $order_id;
}

function o_comment_add()
{
	global $db, $userdata, $lang, $bb_cfg;
	$order_id = (int) $this->request['id'];
	$text = $this->request['text'];
	if (!$text) ajax_die('   ');
//  $text = htmlentities($text);
	$user_id = $userdata['user_id'];
	$time = TIMENOW;
	$text = $db->escape($text);
	$db->query("INSERT INTO bb_order_comment SET
              order_id = $order_id,
              comment = '$text',
              comment_time = $time,
              comment_user_id = $user_id
        ");
	$html  = '<td calss="row1"><a href="profile.php?mode=viewprofile&u='.$user_id.'">'.$userdata['username'].'</a><br />'.create_date($bb_cfg['default_dateformat'], $time, $bb_cfg['board_timezone']).'</td>';
	$html .= '<td calss="row1">'.htmlentities($text).'</td>';
	$this->response['html']  = $html;
	$this->response['id'] = $order_id;
}

}
